# Ilustrarea algoritmului Expectation-Maximization in cazul datelor scalare

# Generare date de test utilizand doua modele gaussiene 
v<-1:1000
for (i in 1:1000){
  z <- sample(c(0,1),1,prob=c(0.7,0.3)) # probabilitati de amestec (mixing probabilities): 0.7 and 0.3
  # probabilitatea ca data sa fie generata folosind primul model este 0.7
  # probabilitatea ca data sa fie generata folosind al doilea model este 0.3
  if (z==0){v[i]<-rnorm(1,mean=1,sd=1); i<-i+1;} # parametrii primului model 1: mean=1, standard deviation=1
  else{v[i]<-rnorm(1,mean=6,sd=2); i<-i+1;}  # parametrii celui de al doilea model: mean=6, standard deviation=2
}

# visualizare distributie date generate + distributia modelului mixt
hist(v,probability=TRUE)
curve(dnorm(x),add=T)
curve(dnorm(x,mean=6,sd=2),add=T)
curve(0.7*dnorm(x,mean=0,sd=1)+0.3*dnorm(x,mean=6,sd=2),add=T,from=-2,to=12,type='b')

# Expectation-Maximization (cazul a 2 clustere)
N <- 100          # dimensiunea setului de date (primele N elemente din v vor fi utilizate pentru a estima modelul)
pi <- c(0.5,0.5)  # estimarea initiala a probabilitatilor de amestec  
mu=c(0,1)         # estimarea initiala a parametrilor modelelor (m1=0, sd1=1, m2=1, sd2=1)
sigma=c(1,1)
r1=1:1000
r2=1:1000
for (iter in 1:50){
# E-step
# calculul "responsabilitatii" corespunzatoare fiecarei date
#  = probabilitatea de a fi fost generata de catre fiecare dintre modele 
#  (folosind valorile curente ale parametrilor modelelor)  

for (i in 1:N){   
     	    	r1[i]<-pi[1]*dnorm(v[i],mu[1],sigma[1])
	        	r2[i]<-pi[2]*dnorm(v[i],mu[2],sigma[2])
            # normalizarea valorilor  
            s <- r1[i] + r2[i]
            r1[i] <- r1[i]/s    
            r2[i] <- r2[i]/s   
	}

# M-step
# estimarea parametrilor modelelor folosind valorile curente ale "responsabilitatilor"
  sr1<-0
	sr2<-0
	for (i in 1:N){
    		sr1<-sr1+r1[i]
   		  sr2<-sr2+r2[i]
	}
	pi[1]<-sr1/N	# estimarea probabilitatilor de mixare pornind de la responsabilitati
	pi[2]<-sr2/N
	mu[1]<-0		  # estimarea mediilor
	mu[2]<-0
	for (i in 1:N){
    	 	mu[1]<-mu[1]+r1[i]*v[i]
   	   	mu[2]<-mu[2]+r2[i]*v[i]
	}
	mu[1]<-mu[1]/sr1
	mu[2]<-mu[2]/sr2

	sigma[1]<-0		# estimarea abaterilor standard
	sigma[2]<-0
	for (i in 1:N){
    	 	sigma[1]<-sigma[1]+r1[i]*(v[i]-mu[1])^2
   	 	  sigma[2]<-sigma[2]+r2[i]*(v[i]-mu[2])^2
	}
	sigma[1]<-sqrt(sigma[1]/sr1)
	sigma[2]<-sqrt(sigma[2]/sr2)
	print("iter=")
	print(iter)
	print("pi=")
	print(pi)
	print("mu=")
	print(mu)
	print("sigma=")
	print(sigma)
	curve(pi[1]*dnorm(x,mean=mu[1],sd=sigma[1])+pi[2]*dnorm(x,mean=mu[2],sd=sigma[2]),add=T,from=-2,to=12,type='b',col=100)
}

# Lab/Tema:  extindeti in cazul datelor bi-dimensionale




