# Biclustering - grupare date atat in raport cu instantele cat si 
#                in raport cu atributele
# Scop: descoperire de sabloane de reglare genica pornind de la date 
#       de tip microarray (nivele de exprimare genica)
# Date: matrice cu M linii si N coloane ce contine valori ale nivelui 
#       de exprimare a M gene pe baza a N esantioane (experimente/ conditii experimentale)

#install.packages("biclust")
library("biclust")

# https://en.wikibooks.org/wiki/Data_Mining_Algorithms_In_R/Clustering/Biclust

# Dataset:
#    Subsample of the Saccharomyces Cerevisiae organism (Yeast).
#    Used to present bicluster algorithms by Barkow et al. (2006)
#    Microarray data: 419 genes, 70 experiments.

data(BicatYeast)
discrYeast <- discretize(BicatYeast)
binYeast <- binarize(BicatYeast,2)

# Biclustering - metode implementate in biclust:
#  BCCC = algoritmul Cheng&Church
#  BCXmotifs = algoritmul Murali&Kasif  (necesita date discretizate)
#              Murali, T. & Kasif, S. Extracting Conserved Gene Expression Motifs from Gene Expression Data Pacific Symposium on Biocomputing, sullivan.bu.edu, 2003, 8, 77-88
#  BCPlaid = algoritm bazat pe Plaid Models (matricea e interpretata ca o "suma de straturi", fiecare strat fiind descris de un model) (http://statweb.stanford.edu/~owen/reports/plaid.pdf) 
#  BCBimax = algoritm care identifica submatrici de 1 intr-o matrice logica - necesita binarizare prealabila a matricii de date (A. Prelic, A systematic comparison and evaluation of biclustering methods
#            for gene expression data, 2006)
#  BCSpectral = algoritm bazat pe descompunerea dupa valori singulare (SVD) - https://www.ncbi.nlm.nih.gov/pmc/articles/PMC430175/

resBCCC <- biclust(BicatYeast, method=BCCC(), delta = 0.1, alpha=1.0, number=50)
resBCCC

## ??? analizati influenta parametrilor delta si alpha supra numarului de biclustere identificate

# vizualizarea primului bicluster - pt a vizualiza alte biclustere se modifica valoarea ultimului parametru
drawHeatmap(BicatYeast,resBCCC,1)
# vizualizarea tuturor biclusterilor 
heatmapBC(x = BicatYeast, bicResult = resBCCC)
# vizualizarea tuturor folosind proiectie bidimensionala
bubbleplot(BicatYeast, bicResult1=resBCCC, bicResult2=NULL, bicResult3=NULL, projection="mean",showLabels=TRUE)
# vizualizarea nivelului de expresie a genelor din cluster 1 (in raport cu nivelul expresiei celorlalte gene)
parallelCoordinates(x=BicatYeast, bicResult=resBCCC, number=1)

##############

resBCXmotifs <- biclust(discrYeast, method=BCXmotifs(), alpha=0.05, number=50)
resBCXmotifs
# vizualizarea unui bicluster (de exemplu: al doilea)
drawHeatmap(discrYeast,resBCXmotifs,2) 
# vizualizarea tuturor biclusterilor 
heatmapBC(x = BicatYeast, bicResult = resBCXmotifs) 
# vizualizarea nivelului de expresie a genelor din cluster 1 (in raport cu nivelul expresiei celorlalte gene)
parallelCoordinates(x=BicatYeast, bicResult=resBCCC, number=1)

##############
resBCPlaid<-biclust(BicatYeast, method=BCPlaid(), verbose=FALSE)
resBCPlaid
# vizualizarea celui de al doilea bicluster
drawHeatmap(BicatYeast,resBCPlaid,2) 
# vizualizarea tuturor biclusterilor 
heatmapBC(x = BicatYeast, bicResult = resBCPlaid)
# vizualizarea nivelului de expresie genica din biclusterul 2 (negru); 
# nivelul corespunzator datelor din afara biclusterului este reprezentat de curbele in gri
parallelCoordinates(x=BicatYeast, bicResult=resBCPlaid, number=2) 

##############
resBCBimax <- biclust(binYeast, method=BCBimax(), minr=4, minc=4, number=50)
resBCBimax
# vizualizarea celui de al doilea bicluster
drawHeatmap(binYeast,resBCBimax,2) 
# vizualizarea tuturor biclusterilor 
heatmapBC(x = BicatYeast, bicResult = resBCBimax)
# nivelul corespunzator datelor din afara biclusterului este reprezentat de curbele in gri
parallelCoordinates(x=BicatYeast, bicResult=resBCBimax, number=2) 

